@@ -21,13 +21,11 @@ import logging
 import operator
 
 from oslo_utils import units
-import six
 
-from django.template.defaultfilters import filesizeformat  # noqa
-from django.utils.text import normalize_newlines  # noqa
+from django.template.defaultfilters import filesizeformat
+from django.utils.text import normalize_newlines
 from django.utils.translation import ugettext_lazy as _
-from django.utils.translation import ungettext_lazy
-from django.views.decorators.debug import sensitive_variables  # noqa
+from django.views.decorators.debug import sensitive_variables
 
 from horizon import exceptions
 from horizon import forms
@@ -42,6 +40,8 @@ from openstack_dashboard.api import cinder
 from openstack_dashboard.api import nova
 from openstack_dashboard.usage import quotas
 
+from openstack_dashboard.dashboards.project.images.images \
+    import tables as image_tables
 from openstack_dashboard.dashboards.project.images \
     import utils as image_utils
 from openstack_dashboard.dashboards.project.instances \
@@ -208,18 +208,16 @@ class SetInstanceDetailsAction(workflows.Action):
         count = cleaned_data.get('count', 1)
 
         # Prevent launching more instances than the quota allows
-        usages = quotas.tenant_quota_usages(self.request)
+        usages = quotas.tenant_quota_usages(
+            self.request,
+            targets=('instances', 'cores', 'ram', 'volumes', ))
         available_count = usages['instances']['available']
         if available_count < count:
-            error_message = ungettext_lazy(
-                'The requested instance cannot be launched as you only '
-                'have %(avail)i of your quota available. ',
-                'The requested %(req)i instances cannot be launched as you '
-                'only have %(avail)i of your quota available.',
-                count)
-            params = {'req': count,
-                      'avail': available_count}
-            raise forms.ValidationError(error_message % params)
+            msg = (_('The requested instance(s) cannot be launched '
+                     'as your quota will be exceeded: Available: '
+                     '%(avail)s, Requested: %(req)s.')
+                   % {'avail': available_count, 'req': count})
+            raise forms.ValidationError(msg)
 
         source_type = cleaned_data.get('source_type')
         if source_type in ('volume_image_id', 'volume_snapshot_id'):
@@ -270,8 +268,9 @@ class SetInstanceDetailsAction(workflows.Action):
             return
         props_mapping = (("min_ram", "ram"), ("min_disk", "disk"))
         for iprop, fprop in props_mapping:
-            if getattr(image, iprop) > 0 and \
-                    getattr(image, iprop) > getattr(flavor, fprop):
+            if (getattr(image, iprop) > 0 and
+                    getattr(flavor, fprop) > 0 and
+                    getattr(image, iprop) > getattr(flavor, fprop)):
                 msg = (_("The flavor '%(flavor)s' is too small "
                          "for requested image.\n"
                          "Minimum requirements: "
@@ -394,9 +393,12 @@ class SetInstanceDetailsAction(workflows.Action):
     def get_help_text(self, extra_context=None):
         extra = {} if extra_context is None else dict(extra_context)
         try:
-            extra['usages'] = api.nova.tenant_absolute_limits(self.request,
-                                                              reserved=True)
+            extra['usages'] = quotas.tenant_quota_usages(
+                self.request,
+                targets=('instances', 'cores', 'ram', 'volumes', 'gigabytes'))
             extra['usages_json'] = json.dumps(extra['usages'])
+            extra['cinder_enabled'] = \
+                base.is_service_enabled(self.request, 'volume')
             flavors = json.dumps([f._info for f in
                                   instance_utils.flavor_list(self.request)])
             extra['flavors'] = flavors
@@ -438,13 +440,15 @@ class SetInstanceDetailsAction(workflows.Action):
                                                   context.get('project_id'),
                                                   self._images_cache)
         for image in images:
-            image.bytes = getattr(image, 'virtual_size', None) or image.size
-            image.volume_size = max(
-                image.min_disk, functions.bytes_to_gigabytes(image.bytes))
-            choices.append((image.id, image))
-            if context.get('image_id') == image.id and \
-                    'volume_size' not in context:
-                context['volume_size'] = image.volume_size
+            if image_tables.get_image_type(image) != "snapshot":
+                image.bytes = getattr(
+                    image, 'virtual_size', None) or image.size
+                image.volume_size = max(
+                    image.min_disk, functions.bytes_to_gigabytes(image.bytes))
+                choices.append((image.id, image))
+                if context.get('image_id') == image.id and \
+                        'volume_size' not in context:
+                    context['volume_size'] = image.volume_size
         if choices:
             choices.sort(key=lambda c: c[1].name or '')
             choices.insert(0, ("", _("Select Image")))
@@ -458,7 +462,7 @@ class SetInstanceDetailsAction(workflows.Action):
                                                   self._images_cache)
         choices = [(image.id, image.name)
                    for image in images
-                   if image.properties.get("image_type", '') == "snapshot"]
+                   if image_tables.get_image_type(image) == "snapshot"]
         if choices:
             choices.sort(key=operator.itemgetter(1))
             choices.insert(0, ("", _("Select Instance Snapshot")))
@@ -488,9 +492,12 @@ class SetInstanceDetailsAction(workflows.Action):
         try:
             if cinder.is_volume_service_enabled(request):
                 available = api.cinder.VOLUME_STATE_AVAILABLE
+                volumes = [v.id for v in cinder.volume_list(
+                           self.request, search_opts=dict(bootable=True))]
                 snapshots = [self._get_volume_display_name(s)
                              for s in cinder.volume_snapshot_list(
-                             self.request, search_opts=dict(status=available))]
+                             self.request, search_opts=dict(status=available))
+                             if s.volume_id in volumes]
         except Exception:
             exceptions.handle(self.request,
                               _('Unable to retrieve list of volume '
@@ -518,8 +525,9 @@ class SetInstanceDetails(workflows.Step):
     def contribute(self, data, context):
         context = super(SetInstanceDetails, self).contribute(data, context)
         # Allow setting the source dynamically.
-        if ("source_type" in context and "source_id" in context
-                and context["source_type"] not in context):
+        if ("source_type" in context and
+                "source_id" in context and
+                context["source_type"] not in context):
             context[context["source_type"]] = context["source_id"]
 
         # Translate form input to context for source values.
@@ -535,7 +543,7 @@ class SetInstanceDetails(workflows.Step):
         return context
 
 
-KEYPAIR_IMPORT_URL = "horizon:project:access_and_security:keypairs:import"
+KEYPAIR_IMPORT_URL = "horizon:project:key_pairs:import"
 
 
 class SetAccessControlsAction(workflows.Action):
@@ -552,6 +560,7 @@ class SetAccessControlsAction(workflows.Action):
         error_messages={'invalid': validators.password_validator_msg()})
     confirm_admin_pass = forms.CharField(
         label=_("Confirm Admin Password"),
+        strip=False,
         required=False,
         widget=forms.PasswordInput(render_value=False))
     groups = forms.MultipleChoiceField(
@@ -582,12 +591,8 @@ class SetAccessControlsAction(workflows.Action):
 
     def populate_groups_choices(self, request, context):
         try:
-            groups = api.network.security_group_list(request)
-            if base.is_service_enabled(request, 'network'):
-                security_group_list = [(sg.id, sg.name) for sg in groups]
-            else:
-                # Nova-Network requires the groups to be listed by name
-                security_group_list = [(sg.name, sg.name) for sg in groups]
+            groups = api.neutron.security_group_list(request)
+            security_group_list = [(sg.id, sg.name) for sg in groups]
         except Exception:
             exceptions.handle(request,
                               _('Unable to retrieve list of security groups'))
@@ -658,9 +663,6 @@ class CustomizeAction(workflows.Action):
             'data-scriptsource-raw': _('Script Data')}),
         required=False)
 
-    def __init__(self, *args):
-        super(CustomizeAction, self).__init__(*args)
-
     def clean(self):
         cleaned = super(CustomizeAction, self).clean()
 
@@ -679,7 +681,7 @@ class CustomizeAction(workflows.Action):
         if has_upload:
             upload_file = files[upload_str]
             log_script_name = upload_file.name
-            LOG.info('got upload %s' % log_script_name)
+            LOG.info('got upload %s', log_script_name)
 
             if upload_file._size > 16 * units.Ki:  # 16kb
                 msg = _('File exceeds maximum size (16kb)')
@@ -688,12 +690,14 @@ class CustomizeAction(workflows.Action):
                 script = upload_file.read()
                 if script != "":
                     try:
+                        if not isinstance(script, str):
+                            script = script.decode()
                         normalize_newlines(script)
                     except Exception as e:
                         msg = _('There was a problem parsing the'
                                 ' %(prefix)s: %(error)s')
                         msg = msg % {'prefix': prefix,
-                                     'error': six.text_type(e)}
+                                     'error': e}
                         raise forms.ValidationError(msg)
                 return script
         else:
@@ -715,24 +719,19 @@ class SetNetworkAction(workflows.Action):
                 " be specified.")},
         help_text=_("Launch instance with"
                     " these networks"))
-    if api.neutron.is_port_profiles_supported():
-        widget = None
-    else:
-        widget = forms.HiddenInput()
-    profile = forms.ChoiceField(label=_("Policy Profiles"),
-                                required=False,
-                                widget=widget,
-                                help_text=_("Launch instance with "
-                                            "this policy profile"))
 
     def __init__(self, request, *args, **kwargs):
         super(SetNetworkAction, self).__init__(request, *args, **kwargs)
+
+        # NOTE(e0ne): we don't need 'required attribute for networks
+        # checkboxes to be able to select only one network
+        # NOTE(e0ne): we need it for compatibility with different
+        # Django versions (prior to 1.11)
+        self.use_required_attribute = False
+
         network_list = self.fields["network"].choices
         if len(network_list) == 1:
             self.fields['network'].initial = [network_list[0][0]]
-        if api.neutron.is_port_profiles_supported():
-            self.fields['profile'].choices = (
-                self.get_policy_profile_choices(request))
 
     class Meta(object):
         name = _("Networking")
@@ -740,34 +739,13 @@ class SetNetworkAction(workflows.Action):
         help_text = _("Select networks for your instance.")
 
     def populate_network_choices(self, request, context):
-        return instance_utils.network_field_data(request)
-
-    def get_policy_profile_choices(self, request):
-        profile_choices = [('', _("Select a profile"))]
-        for profile in self._get_profiles(request, 'policy'):
-            profile_choices.append((profile.id, profile.name))
-        return profile_choices
-
-    def _get_profiles(self, request, type_p):
-        profiles = []
-        try:
-            profiles = api.neutron.profile_list(request, type_p)
-        except Exception:
-            msg = _('Network Profiles could not be retrieved.')
-            exceptions.handle(request, msg)
-        return profiles
+        return instance_utils.network_field_data(request, for_launch=True)
 
 
 class SetNetwork(workflows.Step):
     action_class = SetNetworkAction
-    # Disabling the template drag/drop only in the case port profiles
-    # are used till the issue with the drag/drop affecting the
-    # profile_id detection is fixed.
-    if api.neutron.is_port_profiles_supported():
-        contributes = ("network_id", "profile_id",)
-    else:
-        template_name = "project/instances/_update_networks.html"
-        contributes = ("network_id",)
+    template_name = "project/instances/_update_networks.html"
+    contributes = ("network_id",)
 
     def contribute(self, data, context):
         if data:
@@ -777,9 +755,6 @@ class SetNetwork(workflows.Step):
             networks = [n for n in networks if n != '']
             if networks:
                 context['network_id'] = networks
-
-            if api.neutron.is_port_profiles_supported():
-                context['profile_id'] = data.get('profile', None)
         return context
 
 
@@ -979,13 +954,6 @@ class LaunchInstance(workflows.Workflow):
         if server_group:
             scheduler_hints['group'] = server_group
 
-        port_profiles_supported = api.neutron.is_port_profiles_supported()
-
-        if port_profiles_supported:
-            nics = self.set_network_port_profiles(request,
-                                                  context['network_id'],
-                                                  context['profile_id'])
-
         ports = context.get('ports')
         if ports:
             if nics is None:
@@ -1011,61 +979,16 @@ class LaunchInstance(workflows.Workflow):
                                    scheduler_hints=scheduler_hints)
             return True
         except Exception:
-            if port_profiles_supported:
-                ports_failing_deletes = _cleanup_ports_on_failed_vm_launch(
-                    request, nics)
-                if ports_failing_deletes:
-                    ports_str = ', '.join(ports_failing_deletes)
-                    msg = (_('Port cleanup failed for these port-ids (%s).')
-                           % ports_str)
-                    exceptions.handle(request, msg)
             exceptions.handle(request)
         return False
 
-    def set_network_port_profiles(self, request, net_ids, profile_id):
-        # Create port with Network ID and Port Profile
-        # for the use with the plugin supporting port profiles.
-        nics = []
-        for net_id in net_ids:
-            try:
-                port = api.neutron.port_create(
-                    request,
-                    net_id,
-                    policy_profile_id=profile_id,
-                )
-            except Exception as e:
-                msg = (_('Unable to create port for profile '
-                         '"%(profile_id)s": %(reason)s'),
-                       {'profile_id': profile_id,
-                        'reason': e})
-                for nic in nics:
-                    try:
-                        port_id = nic['port-id']
-                        api.neutron.port_delete(request, port_id)
-                    except Exception:
-                        msg = (msg +
-                               _(' Also failed to delete port %s') % port_id)
-                redirect = self.success_url
-                exceptions.handle(request, msg, redirect=redirect)
-
-            if port:
-                nics.append({"port-id": port.id})
-                LOG.debug("Created Port %(portid)s with "
-                          "network %(netid)s "
-                          "policy profile %(profile_id)s",
-                          {'portid': port.id,
-                           'netid': net_id,
-                           'profile_id': profile_id})
-
-        return nics
-
 
 def _cleanup_ports_on_failed_vm_launch(request, nics):
     ports_failing_deletes = []
     LOG.debug('Cleaning up stale VM ports.')
     for nic in nics:
         try:
-            LOG.debug('Deleting port with id: %s' % nic['port-id'])
+            LOG.debug('Deleting port with id: %s', nic['port-id'])
             api.neutron.port_delete(request, nic['port-id'])
         except Exception:
             ports_failing_deletes.append(nic['port-id'])
