@@ -16,7 +16,7 @@
 import logging
 
 from django.conf import settings
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.utils.translation import ugettext_lazy as _
 import netaddr
 
@@ -28,6 +28,7 @@ from horizon import workflows
 from openstack_dashboard import api
 from openstack_dashboard.dashboards.project.networks.subnets import utils
 from openstack_dashboard import policy
+from openstack_dashboard.utils import settings as setting_utils
 
 
 LOG = logging.getLogger(__name__)
@@ -37,21 +38,11 @@ class CreateNetworkInfoAction(workflows.Action):
     net_name = forms.CharField(max_length=255,
                                label=_("Network Name"),
                                required=False)
-    if api.neutron.is_port_profiles_supported():
-        widget = None
-    else:
-        widget = forms.HiddenInput()
-    net_profile_id = forms.ThemableChoiceField(label=_("Network Profile"),
-                                               required=False,
-                                               widget=widget)
-
-    admin_state = forms.ThemableChoiceField(
-        choices=[(True, _('UP')),
-                 (False, _('DOWN'))],
-        label=_("Admin State"),
+    admin_state = forms.BooleanField(
+        label=_("Enable Admin State"),
+        initial=True,
         required=False,
-        help_text=_("The state to start"
-                    " the network in."))
+        help_text=_("If checked, the network will be enabled."))
     shared = forms.BooleanField(label=_("Shared"), initial=False,
                                 required=False)
     with_subnet = forms.BooleanField(label=_("Create Subnet"),
@@ -68,36 +59,31 @@ class CreateNetworkInfoAction(workflows.Action):
                                      }),
                                      initial=True,
                                      required=False)
+    az_hints = forms.MultipleChoiceField(
+        label=_("Availability Zone Hints"),
+        required=False,
+        help_text=_("Availability zones where the DHCP agents may be "
+                    "scheduled. Leaving this unset is equivalent to "
+                    "selecting all availability zones"))
 
     def __init__(self, request, *args, **kwargs):
         super(CreateNetworkInfoAction, self).__init__(request,
                                                       *args, **kwargs)
-        if api.neutron.is_port_profiles_supported():
-            self.fields['net_profile_id'].choices = (
-                self.get_network_profile_choices(request))
-
         if not policy.check((("network", "create_network:shared"),), request):
-            self.fields['shared'].widget = forms.CheckboxInput(
-                attrs={'disabled': True})
-            self.fields['shared'].help_text = _(
-                'Non admin users are not allowed to set shared option.')
-
-    def get_network_profile_choices(self, request):
-        profile_choices = [('', _("Select a profile"))]
-        for profile in self._get_profiles(request, 'network'):
-            profile_choices.append((profile.id, profile.name))
-        return profile_choices
-
-    def _get_profiles(self, request, type_p):
-        profiles = []
+            self.fields['shared'].widget = forms.HiddenInput()
         try:
-            profiles = api.neutron.profile_list(request, type_p)
+            if api.neutron.is_extension_supported(request,
+                                                  'network_availability_zone'):
+                zones = api.neutron.list_availability_zones(
+                    self.request, 'network', 'available')
+                self.fields['az_hints'].choices = [(zone['name'], zone['name'])
+                                                   for zone in zones]
+            else:
+                del self.fields['az_hints']
         except Exception:
-            msg = _('Network Profiles could not be retrieved.')
-            exceptions.handle(request, msg)
-        return profiles
-    # TODO(absubram): Add ability to view network profile information
-    # in the network detail if a profile is used.
+            msg = _('Failed to get availability zone list.')
+            messages.warning(request, msg)
+            del self.fields['az_hints']
 
     class Meta(object):
         name = _("Network")
@@ -108,8 +94,8 @@ class CreateNetworkInfoAction(workflows.Action):
 
 class CreateNetworkInfo(workflows.Step):
     action_class = CreateNetworkInfoAction
-    contributes = ("net_name", "admin_state", "net_profile_id", "with_subnet",
-                   "shared")
+    contributes = ("net_name", "admin_state", "with_subnet", "shared",
+                   "az_hints")
 
 
 class CreateSubnetInfoAction(workflows.Action):
@@ -124,15 +110,16 @@ class CreateSubnetInfoAction(workflows.Action):
         label=_('Network Address Source'),
         choices=[('manual', _('Enter Network Address manually')),
                  ('subnetpool', _('Allocate Network Address from a pool'))],
-        widget=forms.Select(attrs={
+        widget=forms.ThemableSelectWidget(attrs={
             'class': 'switchable',
             'data-slug': 'source',
         }))
 
     subnetpool = forms.ChoiceField(
         label=_("Address pool"),
-        widget=forms.SelectWidget(attrs={
+        widget=forms.ThemableSelectWidget(attrs={
             'class': 'switched switchable',
+            'data-required-when-shown': 'true',
             'data-slug': 'subnetpool',
             'data-switch-on': 'source',
             'data-source-subnetpool': _('Address pool')},
@@ -145,7 +132,7 @@ class CreateSubnetInfoAction(workflows.Action):
                                 if 'prefixes' in x else "%s" % (x.name)),
         required=False)
 
-    prefixlen = forms.ChoiceField(widget=forms.Select(attrs={
+    prefixlen = forms.ChoiceField(widget=forms.ThemableSelectWidget(attrs={
                                   'class': 'switched',
                                   'data-switch-on': 'subnetpool',
                                   }),
@@ -155,6 +142,10 @@ class CreateSubnetInfoAction(workflows.Action):
     cidr = forms.IPField(label=_("Network Address"),
                          required=False,
                          initial="",
+                         error_messages={
+                             'required': _('Specify "Network Address" or '
+                                           'clear "Create Subnet" checkbox '
+                                           'in previous step.')},
                          widget=forms.TextInput(attrs={
                              'class': 'switched',
                              'data-switch-on': 'source',
@@ -165,7 +156,7 @@ class CreateSubnetInfoAction(workflows.Action):
                          version=forms.IPv4 | forms.IPv6,
                          mask=True)
     ip_version = forms.ChoiceField(choices=[(4, 'IPv4'), (6, 'IPv6')],
-                                   widget=forms.Select(attrs={
+                                   widget=forms.ThemableSelectWidget(attrs={
                                        'class': 'switchable',
                                        'data-slug': 'ipversion',
                                    }),
@@ -215,15 +206,8 @@ class CreateSubnetInfoAction(workflows.Action):
     def __init__(self, request, context, *args, **kwargs):
         super(CreateSubnetInfoAction, self).__init__(request, context, *args,
                                                      **kwargs)
-        if 'with_subnet' in context:
-            self.fields['with_subnet'] = forms.BooleanField(
-                initial=context['with_subnet'],
-                required=False,
-                widget=forms.HiddenInput()
-            )
-
-        if not getattr(settings, 'OPENSTACK_NEUTRON_NETWORK',
-                       {}).get('enable_ipv6', True):
+        if not setting_utils.get_dict_config('OPENSTACK_NEUTRON_NETWORK',
+                                             'enable_ipv6'):
             self.fields['ip_version'].widget = forms.HiddenInput()
             self.fields['ip_version'].initial = 4
 
@@ -248,10 +232,10 @@ class CreateSubnetInfoAction(workflows.Action):
             # Populate data-fields for switching the prefixlen field
             # when user selects a subnetpool other than
             # "Provider default pool"
-            for (id, name) in self.fields['subnetpool'].choices:
-                if not len(id):
+            for (id_, name) in self.fields['subnetpool'].choices:
+                if not id_:
                     continue
-                key = 'data-subnetpool-' + id
+                key = 'data-subnetpool-' + id_
                 self.fields['prefixlen'].widget.attrs[key] = \
                     _('Network Mask')
         else:
@@ -278,7 +262,7 @@ class CreateSubnetInfoAction(workflows.Action):
         if not self.check_subnet_range:
             return
 
-        allowed_cidr = getattr(settings, "ALLOWED_PRIVATE_SUBNET_CIDR", {})
+        allowed_cidr = settings.ALLOWED_PRIVATE_SUBNET_CIDR
         version_str = 'ipv%s' % ip_version
         allowed_ranges = allowed_cidr.get(version_str, [])
         if allowed_ranges:
@@ -292,7 +276,7 @@ class CreateSubnetInfoAction(workflows.Action):
                         'allowed': range_str})
                 raise forms.ValidationError(msg)
 
-    def _check_subnet_data(self, cleaned_data, is_create=True):
+    def _check_subnet_data(self, cleaned_data):
         cidr = cleaned_data.get('cidr')
         ip_version = int(cleaned_data.get('ip_version'))
         gateway_ip = cleaned_data.get('gateway_ip')
@@ -309,7 +293,9 @@ class CreateSubnetInfoAction(workflows.Action):
             msg = _('Specify "Network Address" or '
                     'clear "Create Subnet" checkbox in previous step.')
             raise forms.ValidationError(msg)
-        if cidr:
+        if address_source == 'subnetpool' and 'cidr' in self._errors:
+            del self._errors['cidr']
+        elif cidr:
             subnet = netaddr.IPNetwork(cidr)
             if subnet.version != ip_version:
                 msg = _('Network Address and IP version are inconsistent.')
@@ -325,16 +311,18 @@ class CreateSubnetInfoAction(workflows.Action):
             if netaddr.IPAddress(gateway_ip).version is not ip_version:
                 msg = _('Gateway IP and IP version are inconsistent.')
                 raise forms.ValidationError(msg)
-        if not is_create and not no_gateway and not gateway_ip:
-            msg = _('Specify IP address of gateway or '
-                    'check "Disable Gateway" checkbox.')
-            raise forms.ValidationError(msg)
+        if no_gateway and 'gateway_ip' in self._errors:
+            del self._errors['gateway_ip']
+
+    def _remove_fields_errors(self):
+        self._errors = {}
 
     def clean(self):
-        cleaned_data = super(CreateSubnetInfoAction, self).clean()
-        with_subnet = cleaned_data.get('with_subnet')
+        with_subnet = self.initial.get('with_subnet')
         if not with_subnet:
-            return cleaned_data
+            self._remove_fields_errors()
+            return None
+        cleaned_data = super(CreateSubnetInfoAction, self).clean()
         self._check_subnet_data(cleaned_data)
         return cleaned_data
 
@@ -351,7 +339,7 @@ class CreateSubnetDetailAction(workflows.Action):
                                      initial=True, required=False)
     ipv6_modes = forms.ChoiceField(
         label=_("IPv6 Address Configuration Mode"),
-        widget=forms.Select(attrs={
+        widget=forms.ThemableSelectWidget(attrs={
             'class': 'switched',
             'data-switch-on': 'ipversion',
             'data-ipversion-6': _("IPv6 Address Configuration Mode"),
@@ -394,8 +382,8 @@ class CreateSubnetDetailAction(workflows.Action):
     def __init__(self, request, context, *args, **kwargs):
         super(CreateSubnetDetailAction, self).__init__(request, context,
                                                        *args, **kwargs)
-        if not getattr(settings, 'OPENSTACK_NEUTRON_NETWORK',
-                       {}).get('enable_ipv6', True):
+        if not setting_utils.get_dict_config('OPENSTACK_NEUTRON_NETWORK',
+                                             'enable_ipv6'):
             self.fields['ipv6_modes'].widget = forms.HiddenInput()
 
     def populate_ipv6_modes_choices(self, request, context):
@@ -421,7 +409,7 @@ class CreateSubnetDetailAction(workflows.Action):
             raise forms.ValidationError(msg)
 
     def _check_allocation_pools(self, allocation_pools):
-        for p in allocation_pools.split('\n'):
+        for p in allocation_pools.splitlines():
             p = p.strip()
             if not p:
                 continue
@@ -438,14 +426,14 @@ class CreateSubnetDetailAction(workflows.Action):
                 raise forms.ValidationError(msg)
 
     def _check_dns_nameservers(self, dns_nameservers):
-        for ns in dns_nameservers.split('\n'):
+        for ns in dns_nameservers.splitlines():
             ns = ns.strip()
             if not ns:
                 continue
             self._convert_ip_address(ns, "dns_nameservers")
 
     def _check_host_routes(self, host_routes):
-        for r in host_routes.split('\n'):
+        for r in host_routes.splitlines():
             r = r.strip()
             if not r:
                 continue
@@ -496,20 +484,18 @@ class CreateNetwork(workflows.Workflow):
     def _create_network(self, request, data):
         try:
             params = {'name': data['net_name'],
-                      'admin_state_up': (data['admin_state'] == 'True'),
+                      'admin_state_up': data['admin_state'],
                       'shared': data['shared']}
-            if api.neutron.is_port_profiles_supported():
-                params['net_profile_id'] = data['net_profile_id']
+            if 'az_hints' in data and data['az_hints']:
+                params['availability_zone_hints'] = data['az_hints']
             network = api.neutron.network_create(request, **params)
             self.context['net_id'] = network.id
-            msg = (_('Network "%s" was successfully created.') %
-                   network.name_or_id)
-            LOG.debug(msg)
+            LOG.debug('Network "%s" was successfully created.',
+                      network.name_or_id)
             return network
         except Exception as e:
-            msg = (_('Failed to create network "%(network)s": %(reason)s') %
-                   {"network": data['net_name'], "reason": e})
-            LOG.info(msg)
+            LOG.info('Failed to create network: %s', e)
+            msg = _('Failed to create network "%s".') % data['net_name']
             redirect = self.get_failure_url()
             exceptions.handle(request, msg, redirect=redirect)
             return False
@@ -531,18 +517,18 @@ class CreateNetwork(workflows.Workflow):
                 params['ipv6_address_mode'] = ipv6_modes[1]
         if data['allocation_pools']:
             pools = [dict(zip(['start', 'end'], pool.strip().split(',')))
-                     for pool in data['allocation_pools'].split('\n')
+                     for pool in data['allocation_pools'].splitlines()
                      if pool.strip()]
             params['allocation_pools'] = pools
         if data['host_routes'] or is_update:
             routes = [dict(zip(['destination', 'nexthop'],
                                route.strip().split(',')))
-                      for route in data['host_routes'].split('\n')
+                      for route in data['host_routes'].splitlines()
                       if route.strip()]
             params['host_routes'] = routes
         if data['dns_nameservers'] or is_update:
             nameservers = [ns.strip()
-                           for ns in data['dns_nameservers'].split('\n')
+                           for ns in data['dns_nameservers'].splitlines()
                            if ns.strip()]
             params['dns_nameservers'] = nameservers
 
@@ -567,28 +553,29 @@ class CreateNetwork(workflows.Workflow):
                 params['gateway_ip'] = None
             elif data['gateway_ip']:
                 params['gateway_ip'] = data['gateway_ip']
-            if 'subnetpool' in data and len(data['subnetpool']):
+            if 'subnetpool' in data and data['subnetpool']:
                 params['subnetpool_id'] = data['subnetpool']
-                if 'prefixlen' in data and len(data['prefixlen']):
+                if 'prefixlen' in data and data['prefixlen']:
                     params['prefixlen'] = data['prefixlen']
 
             self._setup_subnet_parameters(params, data)
 
             subnet = api.neutron.subnet_create(request, **params)
             self.context['subnet_id'] = subnet.id
-            msg = _('Subnet "%s" was successfully created.') % data['cidr']
-            LOG.debug(msg)
+            LOG.debug('Subnet "%s" was successfully created.', data['cidr'])
             return subnet
-        except Exception as e:
-            msg = _('Failed to create subnet "%(sub)s" for network "%(net)s": '
-                    ' %(reason)s')
+        except Exception:
+            if network_name:
+                msg = _('Failed to create subnet "%(sub)s" for network '
+                        '"%(net)s".')
+            else:
+                msg = _('Failed to create subnet "%(sub)s".')
             if no_redirect:
                 redirect = None
             else:
                 redirect = self.get_failure_url()
             exceptions.handle(request,
-                              msg % {"sub": data['cidr'], "net": network_name,
-                                     "reason": e},
+                              msg % {"sub": data['cidr'], "net": network_name},
                               redirect=redirect)
             return False
 
@@ -596,15 +583,17 @@ class CreateNetwork(workflows.Workflow):
         """Delete the created network when subnet creation failed."""
         try:
             api.neutron.network_delete(request, network.id)
+            LOG.debug('Delete the created network %s '
+                      'due to subnet creation failure.', network.id)
             msg = _('Delete the created network "%s" '
                     'due to subnet creation failure.') % network.name
-            LOG.debug(msg)
             redirect = self.get_failure_url()
             messages.info(request, msg)
             raise exceptions.Http302(redirect)
-        except Exception:
+        except Exception as e:
+            LOG.info('Failed to delete network %(id)s: %(exc)s',
+                     {'id': network.id, 'exc': e})
             msg = _('Failed to delete network "%s"') % network.name
-            LOG.info(msg)
             redirect = self.get_failure_url()
             exceptions.handle(request, msg, redirect=redirect)
 
@@ -615,7 +604,8 @@ class CreateNetwork(workflows.Workflow):
         # If we do not need to create a subnet, return here.
         if not data['with_subnet']:
             return True
-        subnet = self._create_subnet(request, data, network, no_redirect=True)
+        subnet = self._create_subnet(request, data, network, no_redirect=True,
+                                     tenant_id=network.tenant_id)
         if subnet:
             return True
         else:
